package de.example;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;

class BeanValidationTest {

    private static final String VALID_REQUEST = "{\n"
            + "\"name\":\"Hugo\"\n"
            + "}";

    private static final String INVALID_REQUEST = "{\n"
            + "\"name\":\"123456789010\"\n"
            + "}";

    private static final String EXPECTED_VIOLATION = "[PARAMETER]\r"
            + "[greet.arg0.name]\r"
            + "[value should be between 0 and 10]\r"
            + "[123456789010]\r"
            + "\r";

    @Test
    void buggyBeanValidation() {
        final Client client = ClientBuilder.newClient();
        try {
            final WebTarget greeterTarget = client.target("http://localhost:8080/example/greeter");

            Response response = greeterTarget.request()
                    .post(Entity.entity(INVALID_REQUEST, MediaType.APPLICATION_JSON));
            final String constraintViolation = response.readEntity(String.class);
            // Response is correct
            assertEquals(EXPECTED_VIOLATION, constraintViolation);
            response.close();

            response = greeterTarget.request()
                    .post(Entity.entity(VALID_REQUEST, MediaType.APPLICATION_JSON));
            final String helloHugo = response.readEntity(String.class);
            // Response is correct
            assertEquals("Hello Hugo!", helloHugo);
            response.close();

            response = greeterTarget.request()
                    .post(Entity.entity(INVALID_REQUEST, MediaType.APPLICATION_JSON));
            final String constraintViolation2 = response.readEntity(String.class);
            // Wildfly 13 returns the violation exception instead of the mapped response
            // Wildfly 17.0.1 and 18.Beta1 are duplicating the violation here
            assertEquals(EXPECTED_VIOLATION, constraintViolation2);
            response.close();

            response = greeterTarget.request()
                    .post(Entity.entity(INVALID_REQUEST, MediaType.APPLICATION_JSON));
            final String constraintViolation3 = response.readEntity(String.class);
            // Response is correct
            assertEquals(EXPECTED_VIOLATION, constraintViolation3);
            response.close();
        } finally {
            client.close();
        }
    }
}
